package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.bean.dto.DataPermissionInfo;
import com.qmth.distributed.print.business.bean.dto.DataPermissionQuery;
import com.qmth.distributed.print.business.entity.BasicRoleDataPermission;
import com.qmth.distributed.print.business.mapper.BasicRoleDataPermissionMapper;
import com.qmth.distributed.print.business.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysPrivilege;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.PrivilegeEnum;
import com.qmth.teachcloud.common.service.SysPrivilegeService;
import com.qmth.teachcloud.common.service.SysRoleService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description: 数据权限服务实现类
 * @Author: CaoZixuan
 * @Date: 2022-12-06
 */
@Service
public class BasicRoleDataPermissionServiceImpl extends ServiceImpl<BasicRoleDataPermissionMapper, BasicRoleDataPermission> implements BasicRoleDataPermissionService {
    @Resource
    SysPrivilegeService sysPrivilegeService;
    @Resource
    SysRoleService sysRoleService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveRoleDataPermission(Long roleId, List<DataPermissionInfo> dataPermissionInfoList) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));

        List<Long> privilegeIdList = dataPermissionInfoList.stream()
                .map(DataPermissionInfo::getPrivilegeId)
                .distinct()
                .collect(Collectors.toList());

        if (privilegeIdList.size() > 0) {
            int count = sysPrivilegeService.count(new QueryWrapper<SysPrivilege>()
                    .lambda()
                    .ne(SysPrivilege::getType, PrivilegeEnum.MENU)
                    .in(SysPrivilege::getId, privilegeIdList));

            if (count > 0) {
                throw ExceptionResultEnum.ERROR.exception("角色数据权限要绑在菜单级权限上");
            }
            this.remove(new QueryWrapper<BasicRoleDataPermission>().lambda()
                    .eq(BasicRoleDataPermission::getSchoolId, schoolId)
                    .eq(BasicRoleDataPermission::getRoleId, roleId));

            List<BasicRoleDataPermission> basicRoleDataPermissionList = dataPermissionInfoList.stream().flatMap(e -> {
                BasicRoleDataPermission basicRoleDataPermission = new BasicRoleDataPermission();
                basicRoleDataPermission.insertInfo(requestUser.getId());
                basicRoleDataPermission.setSchoolId(schoolId);
                basicRoleDataPermission.setRoleId(roleId);
                basicRoleDataPermission.setPrivilegeId(e.getPrivilegeId());
                basicRoleDataPermission.setDataPermissionType(e.getDataPermissionType());
                return Stream.of(basicRoleDataPermission);
            }).collect(Collectors.toList());
            this.saveBatch(basicRoleDataPermissionList);
        }

    }

    @Override
    public List<DataPermissionInfo> findRoleDataPermissionInfo(Long roleId) {
        return this.list(new QueryWrapper<BasicRoleDataPermission>().lambda()
                        .eq(BasicRoleDataPermission::getRoleId, roleId))
                .stream()
                .flatMap(e -> {
                    DataPermissionInfo dataPermissionInfo = new DataPermissionInfo();
                    dataPermissionInfo.setPrivilegeId(e.getPrivilegeId());
                    dataPermissionInfo.setDataPermissionType(e.getDataPermissionType());
                    return Stream.of(dataPermissionInfo);
                })
                .distinct()
                .collect(Collectors.toList());
    }

    @Override
    public DataPermissionQuery findDataPermission(Long schoolId, Long requestUserId, String url) {
        // 角色集合
        List<Long> roleList = sysRoleService.getUserRoles(requestUserId);

        SysPrivilege sysPrivilege = sysPrivilegeService.getOne(new QueryWrapper<SysPrivilege>().lambda().eq(SysPrivilege::getUrl, url).eq(SysPrivilege::getType, PrivilegeEnum.URL));
//        sysPrivilege.get
        return null;
    }
}
